import os
import pandas as pd
from io import StringIO
from qnp.Database.RefractiveIndexDatabase.utils import get_df_catalog, get_yaml_db


class RefractiveIndexDatabase:
    PATH = os.path.join(os.path.expanduser("~"), ".QNP", "Databases", "RefractiveIndex")

    def __init__(self):
        self.catalog = get_df_catalog(self.PATH)

    def search(self, material: str = None, article: str = None):
        if material and article:
            query = self.catalog.material.str.contains(
                material
            ) & self.catalog.article.str.contains(article)

        elif material:
            query = self.catalog.material.str.contains(material)

        elif article:
            query = self.catalog.article.str.contains(article)

        else:
            raise BaseException(
                "you have to search with atleast one field: material or(and) article"
            )

        return self.catalog[query]

    def get_data(self, id: int, logs: bool = True):
        material = self.catalog.iloc[id]
        database_path = os.path.join(self.PATH, "data-nk", material.database_path)
        db = get_yaml_db(database_path)

        article = db.get("REFERENCES")
        comments = db.get("COMMENTS", "")

        if len(db.get("DATA")[0]) == 1:
            print("warning... losing some data")
        raw_data = db.get("DATA")[0].get("data")
        df = pd.read_csv(StringIO(raw_data), sep=" ", names=["wavelength", "n", "k"])
        df["epsilon"] = (df.n + df.k * 1j) ** 2

        if logs:
            print(material.material)
            print(article)
            print(comments)

        return df



import matplotlib.pyplot as plt


def twin_plot_decorator(
    title="",
    x_label="",
    ax1_y_label="",
    ax2_y_label="",
    ax1_color="tab:red",
    ax2_color="tab:blue",
):
    def decorator(ploter_function):
        def wrapper():
            fig, ax1 = plt.subplots()
            plt.title(title)

            ax1.set_xlabel(x_label)
            ax1.set_ylabel(ax1_y_label, color=ax1_color)
            ax1.tick_params(axis="y", labelcolor=ax1_color)

            # instantiate a second Axes that shares the same x-axis
            ax2 = ax1.twinx()

            ax2.set_ylabel(ax2_y_label, color=ax2_color)
            ax2.tick_params(axis="y", labelcolor=ax2_color)

            fig.tight_layout()

            ploter_function(ax1, ax2)

            ax1.legend()
            ax2.legend()

        wrapper()

    return decorator



import numpy as np
from qnp.Physics.Constants import Plancks_Constant, Speed_of_Light, pi
from typing import Union


def frequency(wavelength_um: Union[np.ndarray, float]) -> Union[np.ndarray, float]:
    """get wavelength in [um] and return frequency in [Hz]"""

    return Speed_of_Light.MperS / (wavelength_um * 1e-6)


def omega(wavelength_um: Union[np.ndarray, float]) -> Union[np.ndarray, float]:
    """get wavelength in [um] and return omega in [rad/s]"""

    return 2 * pi * frequency(wavelength_um)


class Energy:
    def __init__(self, wavelength_um: Union[np.ndarray, float]):
        self.frequency = frequency(wavelength_um)

    def joule(self) -> Union[np.ndarray, float]:
        """return Energy in [J] unit"""

        return Plancks_Constant.JS * self.frequency

    def eV(self) -> Union[np.ndarray, float]:
        """return Energy in [eV] unit"""

        return Plancks_Constant.eVS * self.frequency



import os
import urllib.request
import zipfile


class RefractiveIndexDatabase:
    _DATABASE_SHA = "451b9136b4b3566f6259b703990add5440ca125f"
    DB_URL = f"https://github.com/polyanskiy/refractiveindex.info-database/archive/{_DATABASE_SHA}.zip"

    BASE_PATH = os.path.join(
        os.path.expanduser("~"), ".QNP", "Databases", "RefractiveIndex"
    )
    ZIP_PATH = os.path.join(BASE_PATH, "RefractiveIndex.zip")

    def download_db(self):
        # Create the directories if they don't exist
        os.makedirs(self.BASE_PATH, exist_ok=True)

        # Download the ZIP file if necessary
        if not os.listdir(self.BASE_PATH):
            print(f"Downloading database from {self.DB_URL}...")
            urllib.request.urlretrieve(self.DB_URL, self.ZIP_PATH)
            print(f"Database downloaded to: {self.ZIP_PATH}")

            # Unzip and move contents
            self.unzip_db()
        else:
            print("Database already exists. Skipping download.")

    def unzip_db(self):
        print(f"Unzipping database contents to: {self.BASE_PATH}...")
        try:
            with zipfile.ZipFile(self.ZIP_PATH, "r") as zip_ref:
                # Extract only the "database/" folder
                for member in zip_ref.namelist():
                    if member.startswith(
                        f"refractiveindex.info-database-{self._DATABASE_SHA}/database/"
                    ):
                        # Remove the ZIP folder structure to place files directly
                        relative_path = member.split("database/")[1]
                        target_path = os.path.join(self.BASE_PATH, relative_path)

                        if member.endswith("/"):
                            # If it's a folder, create it
                            os.makedirs(target_path, exist_ok=True)
                        else:
                            # If it's a file, extract it
                            os.makedirs(os.path.dirname(target_path), exist_ok=True)
                            with (
                                zip_ref.open(member) as source,
                                open(target_path, "wb") as target,
                            ):
                                target.write(source.read())

            print(f"Database contents extracted to: {self.BASE_PATH}")

            # Remove the ZIP file after extraction
            os.remove(self.ZIP_PATH)
            print(f"Deleted ZIP file: {self.ZIP_PATH}")

        except zipfile.BadZipFile:
            raise BaseException(
                "Failed to unzip the database. The file may be corrupted."
            )


if __name__ == "__main__":
    RefractiveIndexDatabase().download_db()
